<#-- @formatter:off -->
<#include "../mcitems.ftl">
{
    <#if data.recipeShapeless>
        "type": "minecraft:crafting_shapeless",
    <#else>
        "type": "minecraft:crafting_shaped",
    </#if>
    <#if data.group?has_content>"group": "${data.group}",</#if>
    "category": "${data.craftingBookCategory?lower_case}",
    <#if data.recipeShapeless>
        "ingredients": [
          <#assign ingredients = "">
          <#list data.recipeSlots as element>
              <#if !element.isEmpty()>
                  <#assign ingredients += "{${mappedMCItemToItemObjectJSON(element)}},">
              </#if>
          </#list>
            ${ingredients[0..(ingredients?last_index_of(',') - 1)]}
        ],
    <#else>
        <#assign recipeData = buildRecipeMatrix()>
        <#assign recipeArray = recipeData[0], recipeMappings = recipeData[1]>
        "pattern": [
        <#list recipeArray as arrayRow>
            "${arrayRow?join("")}"<#sep>,
        </#list>
        ],
        "key": {
        <#list recipeMappings as mappingKey, mappingValue>
            "${mappingKey}": {${mappedMCItemToItemObjectJSON(mappingValue)}}<#sep>,
        </#list>
        },
    </#if>
    "result": {
        ${mappedMCItemToItemObjectJSON(data.recipeReturnStack)},
        "count": ${data.recipeRetstackSize}
    }
}

<#--
 # This function processes returned optimized recipe data, removing
 # any duplicate mappings and thus making it look like vanilla recipes.
 # 65th and following ingredients will be ignored due to lack of supported symbols.
 #
 # Copyright (C) 2021-2023, Defeatomizer
-->
<#function buildRecipeMatrix>
	<#local matrix = [], mappings = {}>
	<#list data.getOptimisedRecipe() as row>
		<#local rowNew = []>
		<#list row as mcItem>
			<#if mcItem.isEmpty()> <#-- Empty slot. -->
				<#local rowNew += [' ']>
			<#elseif mappings?values?seq_contains(mcItem)> <#-- Item is already mapped. -->
				<#local rowNew += [mappings?keys?filter(e -> mappings[e] == mcItem)[0]]>
			<#else> <#-- No mapping for this item yet. -->
				<#local str = generator.map(mcItem.getUnmappedValue(), "blocksitems", 1)>
				<#if str?contains(":")> <#-- If the result string contains namespaces, they should be removed. -->
					<#local str = str.substring(str?last_index_of(":") + 1)>
				</#if>
				<#local chars = str.replaceAll("[^A-Za-z\\d]", "").toUpperCase()
						.concat("ABCDEFGHIJKLMNOPQRSTUVWXYZ!#$%&()+,-./:;<=>@[\\]^_`{|}~0123456789 ").toCharArray()>
				<#list chars as chr>
					<#if !mappings[chr]??> <#-- If a char not taken as a key earlier is found, it gets assigned. -->
						<#local rowNew += [chr], mappings += (chr != ' ')?then({chr: mcItem}, {})>
						<#break>
					</#if>
				</#list>
			</#if>
		</#list>
		<#local matrix += [rowNew]>
	</#list>
	<#return [matrix, mappings]>
</#function>

<#-- @formatter:on -->